extends TileMap

const min_x_width: int = 10;
const max_x_width: int = 35;
const x_margin: int = 20;
const y_prerender: int = 10;

var last_tile_y : int = -10;
var x_offset : int = 0;
var x_width : int = 20;
var last_had_gs : bool = false;

var last_line : Array[TEXT_TYPE] = []
var new_line : Array[TEXT_TYPE] = []


enum TEXT_TYPE {
	TREE, POWDER, 
	L_EDGE_GROW, L_GROW_EDGE, 
	L_EDGE_SHRINK, L_SHRINK_EDGE,
	L_EDGE, SLOPE, R_EDGE,
	R_EDGE_GROW, R_GROW_EDGE,
	R_EDGE_SHRINK, R_SHRINK_EDGE,
	NONE
}


func get_texture(tt: TEXT_TYPE) -> Vector2i:
	match tt:
		TEXT_TYPE.TREE:
			if(randf()>0.9):
				return Vector2i(6,0)
			return Vector2i(6,2)
		TEXT_TYPE.POWDER:
			if(randf()>0.2):
				return Vector2i(0,0)
			return Vector2i(5,0)
		
		TEXT_TYPE.L_EDGE_GROW:
			return Vector2i(2,1)
		TEXT_TYPE.L_GROW_EDGE:
			return Vector2i(0,1)
		
		TEXT_TYPE.L_EDGE_SHRINK:
			return Vector2i(2,4)
		TEXT_TYPE.L_SHRINK_EDGE:
			return Vector2i(0,2)
		
		TEXT_TYPE.L_EDGE:
			return Vector2i(1,0)
		
		TEXT_TYPE.SLOPE:
			if(randf()>0.2):
				return Vector2i(2,0)
			return Vector2i(3,0)

		TEXT_TYPE.R_EDGE:
			return Vector2i(4,0)
		
		TEXT_TYPE.R_EDGE_GROW:
			return Vector2i(3,1)
		TEXT_TYPE.R_GROW_EDGE:
			return Vector2i(5,1)
		
		TEXT_TYPE.R_EDGE_SHRINK:
			return Vector2i(3,4)
		TEXT_TYPE.R_SHRINK_EDGE:
			return Vector2i(5,2)
		_:
			return Vector2i(0,0)

func gen_line():
	if(last_line.size() == 0):
		new_line.resize(x_width+ 2*x_margin)
		new_line.fill(TEXT_TYPE.POWDER)
		for j in range(x_margin+1, x_margin+x_width ):
			new_line[j] = TEXT_TYPE.SLOPE

		new_line[x_margin] = TEXT_TYPE.L_EDGE
		new_line[x_margin+x_width] = TEXT_TYPE.R_EDGE
		last_line = Array(new_line)
		return

	var last_width = x_width
	var grow = !last_had_gs && (last_width<max_x_width) && randf()<0.3
	var shrink = !last_had_gs && (last_width>min_x_width) && randf()<0.3
	last_had_gs = grow || shrink

	var grow_rand = randf()
	var shrink_rand = randf()

	var grow_l = grow && grow_rand<0.6
	var shrink_l = shrink && shrink_rand<0.6
	var grow_r = grow && grow_rand>=0.6
	var shrink_r = shrink && shrink_rand>=0.6

	var start_offset = -1 if grow_l else (1 if shrink_l else 0)
	var end_offset = 1 if grow_r else (-1 if shrink_r else 0)
	var this_width = last_width + (-start_offset) + end_offset
	var this_offset = x_offset + start_offset
	
	new_line.resize(this_width+ 2*x_margin)
	new_line.fill(TEXT_TYPE.POWDER)
	
	for j in range(x_margin, x_margin+this_width ):
		new_line[j] = TEXT_TYPE.SLOPE
		
	if(grow_l):
		new_line[x_margin] = TEXT_TYPE.POWDER
		new_line[x_margin+1] = TEXT_TYPE.L_EDGE_GROW
	elif(shrink_l):
		new_line[x_margin] = TEXT_TYPE.SLOPE
		new_line[x_margin-1] = TEXT_TYPE.L_SHRINK_EDGE
	else:
		if(last_line[x_margin+1] == TEXT_TYPE.L_EDGE_GROW):
			new_line[x_margin] = TEXT_TYPE.L_GROW_EDGE
		elif(last_line[x_margin-1] == TEXT_TYPE.L_SHRINK_EDGE):
			new_line[x_margin] = TEXT_TYPE.L_EDGE_SHRINK
		else:
			new_line[x_margin] = TEXT_TYPE.L_EDGE
	
	if(grow_r):
		new_line[x_margin+this_width] = TEXT_TYPE.POWDER
		new_line[x_margin+this_width-1] = TEXT_TYPE.R_EDGE_GROW
	elif(shrink_r):
		new_line[x_margin+this_width] = TEXT_TYPE.SLOPE
		new_line[x_margin+this_width+1] = TEXT_TYPE.R_SHRINK_EDGE
	else:
		if(last_line[x_margin+this_width-1] == TEXT_TYPE.R_EDGE_GROW):
			new_line[x_margin+this_width] = TEXT_TYPE.R_GROW_EDGE
		elif(last_line[x_margin+this_width+1] == TEXT_TYPE.R_SHRINK_EDGE):
			new_line[x_margin+this_width] = TEXT_TYPE.R_EDGE_SHRINK
		else:
			new_line[x_margin+this_width] = TEXT_TYPE.R_EDGE
	
	x_offset = this_offset
	x_width = this_width
	last_line = new_line.duplicate()#Array.from(new_line)

func gen_cells(ny:int):
	for i in range(last_tile_y, last_tile_y+ny):
		gen_line()
		for j in range(last_line.size()):
			set_cell(0,Vector2(j+x_offset,i), 0,get_texture(last_line[j]),0)
	last_tile_y += ny

func _ready():
	Global.map = self
	add_child(Global.player)
	Global.player.position=Vector2((x_offset + x_margin + x_width/2.0)*Global.TILE_SCALE,0)
	while(Global.player.position.y + y_prerender*Global.TILE_SCALE > last_tile_y*Global.TILE_SCALE):
		gen_cells(2)
	add_child(Global.avalanche)
	pass 

func reset():
	last_tile_y = -10;
	x_offset = 0;
	x_width = 20;
	Global.player.position=Vector2((x_offset + x_margin + x_width/2.0)*Global.TILE_SCALE,0)
	Global.avalanche.reset()
	
	last_line = []
	new_line = []
	clear()
	while(Global.player.position.y + y_prerender*Global.TILE_SCALE > last_tile_y*Global.TILE_SCALE):
		gen_cells(2)

func _process(delta):
	if(Global.player.position.y + y_prerender*Global.TILE_SCALE > last_tile_y*Global.TILE_SCALE):
		gen_cells(1)
		if(randf()<0.2):
			var new_enemy = Global.enemy.instantiate()
			new_enemy.position.y = last_tile_y*Global.TILE_SCALE
			new_enemy.position.x = (x_offset+x_margin + randf()*x_width)*Global.TILE_SCALE
			add_child(new_enemy)



