class_name Player
extends CharacterBody2D

var angle : float = 0
var breaking : bool = false
@export var energy = 0
@export var mass = 50

var score = 0
var coin = 0

var multiplier = 0;


var gravity = ProjectSettings.get_setting("physics/2d/default_gravity")
var carve_angle : float = PI/1.8

func _ready():
	pass
	
func reset():
	energy = 0
	score = 0
	coin = 0
	mass = 50
	angle = 0
	carve_angle = PI/1.8

func _physics_process(delta):
	var direction = get_global_mouse_position()-position
	var new_angle : float = direction.angle()-PI/2
	if(new_angle >= PI):
		new_angle -= 2*PI
	if(new_angle <= -PI):
		new_angle += 2*PI

	if(abs(new_angle-angle)>PI/8):
		new_angle = lerp(angle, new_angle, 0.05)
	angle = new_angle
	rotation = angle

	var delta_height = sin(atan(Global.slope)) * sin(angle+PI/2) * delta
	energy += mass * gravity * delta_height * Global.TILE_SCALE
	if(energy <= 0): energy = 0

	var speed =  sqrt(2*energy/mass)
	var friction = 0.5* (speed**2) * delta
	breaking = Input.is_key_pressed(KEY_SPACE)
	if(Input.is_key_pressed(KEY_SPACE)):
		friction += max(10* (speed**2) * delta,1.2*1e3)
		if(angle > 0):
			rotation -= PI/2
		else:
			rotation += PI/2
	
	energy -= 0.5 * friction**2 / mass
	if(energy <= 0):
		energy = 0
	var f_speed =  sqrt(2*energy/mass)
	var kmh_speed = f_speed*1000 / 3600
	
	$skin.scale.x = -1 if(angle > 0) else 1
	$speed.visible = !breaking && kmh_speed > 50
	$speed.play()
	$break.visible = breaking && kmh_speed > 15
	$break.play()
	$break.position.x = -10 if angle>0 else +10
	$break.scale.x = -1 if angle>0 else +1
	
	var new_vy = f_speed * sin(angle+PI/2)
	var new_vx = f_speed * cos(angle+PI/2)
	
	velocity.y = new_vy
	velocity.x = new_vx
	
	$Control/BotRight/PlayerSpeed.text = "Player: "+ str(int(kmh_speed))+"km/h"
	$Control/TopLeft/Slope.text = "Slope: "+ str(int(Global.slope*100))+"%"
	$Control/TopLeft/AvalancheSpeed.text = "Avalanche: "+  str(int(Global.avalanche.kmh_speed))+"km/h"
	$Control/TopRight/Score.text = "Score: "+str(int(score))
	$Control/TopRight/Coin.text = "Gold: "+str(int(coin))
	
	move_and_slide()
	for i in range(get_slide_collision_count()):
		var curcol = get_slide_collision(i)
		if(curcol.get_collider().is_in_group("Enemy")):
			pass
		else:
			var wall_friction = max(2* (speed**2) * delta,1e3)
			energy -= 0.5 * wall_friction**2 / mass
			if(energy<0): 
				energy = 0

func ski_col_side(col_angle):
	var ski_angle_col = rotation-col_angle-PI/2
	if(ski_angle_col<0): ski_angle_col += 2*PI
	
	if(ski_angle_col>(PI/2-carve_angle/2) and ski_angle_col<(PI/2+carve_angle/2)):
		score += 10
		multiplier = 2.0;
		pass
	elif(ski_angle_col>(3*PI/2-carve_angle/2) and ski_angle_col<(3*PI/2+carve_angle/2)):
		score += 10
		multiplier = 2.0
		pass
	else:
		var enemy_hit_friction = 1e4
		multiplier = 0
		score += 1
		energy -= 0.5 * enemy_hit_friction**2 / mass
		if(energy<0): 
			energy = 0

func _on_entered(body):
	if(body.is_in_group("Enemy")):
		ski_col_side(position.angle_to_point(body.position))
		coin += 1
		body.queue_free()



func _on_area_entered(area):
	if(area.is_in_group("Avalanche")):
		Global.reset()
